<?php
/**
 * BSS Commerce Co.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://bsscommerce.com/Bss-Commerce-License.txt
 *
 * @category   BSS
 * @package    Bss_DcoMigrate
 * @author     Extension Team
 * @copyright  Copyright (c) 2020-2021 BSS Commerce Co. ( http://bsscommerce.com )
 * @license    http://bsscommerce.com/Bss-Commerce-License.txt
 */

namespace Bss\DcoMigrate\Console\Command;

use Magento\Framework\App\ObjectManager;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;

class DataMigration extends Command
{
    const LIMIT = 'limit';

    /**
     * @inheritDoc
     */
    protected function configure()
    {
        $this->setName('dco:migration');
        $this->setDescription('This is a command which migrate old dependent custom option data (v1.0.4) to new dependent custom option data(>= v1.0.9) (Bss_DependentCustomOption). Please back up table bss_depend_co before do this action.');
        $this->addOption(
            self::LIMIT,
            null,
            InputOption::VALUE_OPTIONAL,
            'Limit Product'
        );

        parent::configure();
    }

    /**
     * Execute the command
     *
     * @param InputInterface $input
     * @param OutputInterface $output
     *
     * @return null|int
     */
    protected function execute(InputInterface $input, OutputInterface $output)
    {
        /** @var \Magento\Framework\Module\Manager $moduleManager */
        $moduleManager = ObjectManager::getInstance()->create(\Magento\Framework\Module\Manager::class);
        if (!$moduleManager->isEnabled('Bss_DependentCustomOption')) {
            $output->writeln('<info>Module `Bss_DependentCustomOption` is disabled. Skip this action.</info>');
        } else {
            $limit = $input->getOption(self::LIMIT);
            if (!$limit || (int)$limit <= 0) {
                $limit = 0;
                $output->writeln('<info>Limit product params not be set. The migration action is processing all data in bss_depend_co table.</info>');
            }
            $output->writeln('<info>Migrate Action Start.</info>');
            $dcoResource = ObjectManager::getInstance()->create(\Bss\DcoMigrate\Model\ResourceModel\Dco::class);
            $rows = $dcoResource->migrate($limit);
            $output->writeln('<info>Done: ' . (int)$rows . ' rows.</info>');
        }
    }
}
