<?php
/**
 * BSS Commerce Co.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://bsscommerce.com/Bss-Commerce-License.txt
 *
 * @category   BSS
 * @package    Bss_DcoMigrate
 * @author     Extension Team
 * @copyright  Copyright (c) 2020-2021 BSS Commerce Co. ( http://bsscommerce.com )
 * @license    http://bsscommerce.com/Bss-Commerce-License.txt
 */

namespace Bss\DcoMigrate\Model\ResourceModel;

use Bss\DependentCustomOption\Model\ResourceModel\DependOption;
use Magento\Framework\Exception\LocalizedException;

class Dco extends DependOption
{
    /**
     * @param int $limitProduct
     */
    public function migrate($limitProduct = 0)
    {
        // Get list all dco in db
        $select = $this->getConnection()->select();
        $optionTable = $this->_resources->getTableName('bss_depend_co');
        $select->from(
            ['optionTable' => $optionTable],
            ['increment_id', 'depend_value', 'option_type_id', 'option_id', 'product_id']
        );

        $oldDcoList = $this->getConnection()->fetchAll($select);

        // Get product by OptionID
        $optionIds = [];
        $optionTypeIds = [];
        foreach ($oldDcoList as $oldDcoItem) {
            if (!in_array($oldDcoItem['option_id'], $optionIds)) {
                $optionIds[] = $oldDcoItem['option_id'];
            }
        }

        foreach ($oldDcoList as $oldDcoItem) {
            if (!in_array($oldDcoItem['option_type_id'], $optionTypeIds)) {
                $optionTypeIds[] = $oldDcoItem['option_type_id'];
            }
        }
        $optionTypeList = $this->getOptionTypeList($optionTypeIds);
        foreach ($optionTypeList as $optionTypeItem) {
            if (isset($optionTypeItem['option_id']) && !in_array($optionTypeItem['option_id'], $optionIds)) {
                $optionIds[] = $optionTypeItem['option_id'];
            }
        }
        $optionListItems = $this->getOptionListItems($optionIds);

        // Process data
        $oldDcoData = [];
        foreach ($optionListItems as $optionItem) {
            if (!in_array($optionItem['product_id'], $oldDcoData)) {
                $oldDcoData[] = $optionItem['product_id'];
            }
        }

        $processAllThis = $oldDcoData;
        if ($limitProduct !== 0 && $limitProduct > 0) {
            $processAllThis = array_slice($oldDcoData, 0, $limitProduct);
        }

        $rows = 0;
        foreach ($processAllThis as $productId) {
            $rows += $this->saveNewDcoData($productId);
        }
        return $rows;
    }

    /**
     * @param array $optionTypeIds
     * @return array
     */
    protected function getOptionTypeList($optionTypeIds)
    {
        $select = $this->getConnection()->select();
        $optionTypeTable = $this->_resources->getTableName('catalog_product_option_type_value');
        $select->from(
            ['optionTypeTable' => $optionTypeTable],
            ['option_type_id', 'option_id']
        )->where(
            'optionTypeTable.option_type_id IN (?)', $optionTypeIds
        );
        return $this->getConnection()->fetchAll($select);
    }

    /**
     * @param array $optionIds
     * @return array
     */
    protected function getOptionListItems($optionIds)
    {
        $select = $this->getConnection()->select();
        $optionTypeTable = $this->_resources->getTableName('catalog_product_option');
        $select->from(
            ['optionTable' => $optionTypeTable],
            ['option_id', 'product_id']
        )->where(
            'optionTable.option_id IN (?)', $optionIds
        );
        return $this->getConnection()->fetchAll($select);
    }

    /**
     * @param int $productId
     * @throws \Exception
     */
    public function saveNewDcoData($productId)
    {
        // Get all options
        $allOptions = $this->getAllOptionIdsByProduct($productId);
        $boilerplateList = [];

        // Remain old dco values
        $oldDcoList = $this->getDcoByProduct($productId);
        // Remove old dco of product
        $this->removeOldDco($productId, $allOptions);

        $lastIncrementProduct = 1;
        // Get options dco data to insert
        foreach ($allOptions as $option) {
            if (isset($option['option_id']) && $option['option_id'] && !isset($boilerplateList[$option['option_id']])) {

                $boilerplateOption = $this->createBoilerplateDco(
                    $lastIncrementProduct,
                    $productId,
                    $option['option_id'],
                    NULL,
                    1
                );

                // Get option values dco data to insert
                foreach ($allOptions as $optionAnother) {
                    if (!isset($boilerplateList[$option['option_id']][$optionAnother['option_type_id']]) &&
                        isset($optionAnother['option_type_id']) && $optionAnother['option_type_id'] &&
                        isset($optionAnother['option_id']) && $optionAnother['option_id'] &&
                        $optionAnother['option_id'] == $option['option_id']) {
                        $boilerplateOptionType = $this->createBoilerplateDco(
                            $lastIncrementProduct,
                            $productId,
                            $optionAnother['option_type_id'],
                            NULL,
                            2
                        );
                        $boilerplateList[$option['option_id']][] = $boilerplateOptionType;
                    }
                }

                // Re-sort list
                $boilerplateList[$option['option_id']][] = $boilerplateOption;

            }
        }
        // Get all list data will be inserted
        $list = [];
        foreach ($boilerplateList as $boilerplate) {
            foreach ($boilerplate as $item) {
                $list[] = $item;
            }
        }

        // Save new dco after remove old dco
        $check = [];
        foreach ($oldDcoList as $oldDcoItem) {
            $oldDataDependOnId = $oldDcoItem['depend_value'] ? explode(',', $oldDcoItem['depend_value']) : null;
            if ($oldDataDependOnId && !empty($oldDataDependOnId)) {
                foreach ($oldDcoList as $oldDcoItemNext) {
                    if (in_array($oldDcoItemNext['increment_id'], $oldDataDependOnId)) {
                        $valueOption = $oldDcoItem['option_id'];
                        $valueOptionType = $oldDcoItem['option_type_id'];
                        $dependOption = $oldDcoItemNext['option_id'];
                        $dependOptionType = $oldDcoItemNext['option_type_id'];
                        $check[] = [
                            'value' => $valueOptionType ? $valueOptionType : $valueOption,
                            'depend_on' => $dependOptionType ? $dependOptionType : $dependOption,
                        ];
                    }
                }
            }
        }

        foreach ($list as &$item) {
            $dependOfItem = [];
            foreach ($check as $itemCheck) {
                $value = $itemCheck['value'];
                $depend = $itemCheck['depend_on'];
                $itemOption = $item['option_id'];
                $itemType = $item['option_type_id'];
                if ($value == $itemOption || $value == $itemType) {
                    foreach ($list as $itemNext) {
                        $itemNextOption = $itemNext['option_id'];
                        $itemNextType = $itemNext['option_type_id'];
                        if ($depend == $itemNextOption || $depend == $itemNextType) {
                            $dependOfItem[] = $itemNext['increment_id'];
                        }
                    }
                }
            }
            if (!empty($dependOfItem)) {
                asort($dependOfItem);
                $item['depend_value'] = implode(',', $dependOfItem);
            } else {
                $item['depend_value'] = NULL;
            }
        }

        $this->saveNewDcos($list);
        return count($list);
    }

    /**
     * @param $incrementId
     * @param $productId
     * @param $optionId
     * @param $dependValue
     * @param int $type = 1 is option_id 2 is option_type_id
     * @return array
     */
    protected function createBoilerplateDco(
        &$incrementId,
        $productId,
        $optionId,
        $dependValue = NULL,
        $type = 1
    ) {
        $boilerplate = [
            'increment_id' => $incrementId,
            'product_id' => $productId,
            'depend_value' => $dependValue
        ];
        if ($type === 1) {
            $boilerplate['option_id'] = $optionId;
            $boilerplate['option_type_id'] = NULL;
        } elseif ($type === 2) {
            $boilerplate['option_id'] = NULL;
            $boilerplate['option_type_id'] = $optionId;
        }
        $incrementId++;
        return $boilerplate;
    }

    /**
     * @param array $dcoItems
     * @return $this
     * @throws LocalizedException
     */
    protected function saveNewDcos($dcoItems) {
        $this->getConnection()->insertMultiple($this->getMainTable(), $dcoItems);

        return $this;
    }

    /**
     * @param int|string $productId
     * @return array
     */
    protected function getAllOptionIdsByProduct($productId)
    {
        $options = $this->getOptionsByProduct($productId);
        $optionValues = $this->getOptionValuesByProduct($productId);
        $optionHasValues = [
            'checkbox',
            'drop_down',
            'radio',
            'multiple'
        ];
        $result = [];
        foreach ($options as $option) {
            if (!in_array($option['type'], $optionHasValues)) {
                $result[] = [
                    'option_id' => $option['option_id'],
                    'option_type_id' => null
                ];
            } else {
                foreach ($optionValues as $optionValue) {
                    if ($option['option_id'] == $optionValue['option_id']) {
                        $result[] = [
                            'option_id' => $option['option_id'],
                            'option_type_id' => $optionValue['option_type_id']
                        ];
                    }
                }
            }
        }

        return $result;
    }

    /**
     * @param int|string $productId
     * @return array
     */
    protected function getOptionsByProduct($productId)
    {
        $select = $this->getConnection()->select();
        $optionTable = $this->_resources->getTableName('catalog_product_option');
        $select->from(
            ['optionTable' => $optionTable],
            ['option_id', 'type']
        )->where('optionTable.product_id = ?', $productId);

        return $this->getConnection()->fetchAll($select);
    }

    /**
     * @param int|string $productId
     * @return array
     */
    protected function getOptionValuesByProduct($productId)
    {
        $select = $this->getConnection()->select();
        $mainTable = $this->_resources->getTableName('catalog_product_option');
        $subTable = $this->_resources->getTableName('catalog_product_option_type_value');

        $select->from(
            ['valueTable' => $subTable],
            [
                'option_type_id',
                'option_id'
            ]
        )->joinInner(
            ['optionTable' => $mainTable],
            'valueTable.option_id=optionTable.option_id',
            ''
        )->where('optionTable.product_id = ?', $productId);

        return $this->getConnection()->fetchAll($select);
    }

    /**
     * @param int $productId
     * @param array $options
     * @throws LocalizedException
     */
    protected function removeOldDco($productId, $options)
    {
        $optionIds = [];
        $optionTypeIds = [];
        foreach ($options as $option) {
            if (isset($option['option_id']) && $option['option_id']) {
                $optionIds[] = $option['option_id'];
            }
            if (isset($option['option_type_id']) && $option['option_type_id']) {
                $optionTypeIds[] = $option['option_type_id'];
            }
        }

        $this->getConnection()->delete(
            $this->getMainTable(),
            [
                'option_id IN (?)' => $optionIds
            ]
        );
        $this->getConnection()->delete(
            $this->getMainTable(),
            [
                'option_type_id IN (?)' => $optionTypeIds
            ]
        );
    }

    /**
     * @param int|string $productId
     * @return array
     * @throws LocalizedException
     */
    public function getDcoByProduct($productId)
    {
        $select = $this->getConnection()->select()->from(
            ['main_table' => $this->getMainTable()],
            ['option_type_id', 'option_id', 'product_id', 'depend_value', 'increment_id']
        )->where(
            'main_table.product_id = ?',
            $productId
        );
        return $this->getConnection()->fetchAll($select);
    }
}
