<?php
/**
 * BSS Commerce Co.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://bsscommerce.com/Bss-Commerce-License.txt
 *
 * @category   BSS
 * @package    Bss_DependentCustomOption
 * @author     Extension Team
 * @copyright  Copyright (c) 2017-2020 BSS Commerce Co. ( http://bsscommerce.com )
 * @license    http://bsscommerce.com/Bss-Commerce-License.txt
 */
namespace Bss\DependentCustomOption\Observer;

use Magento\Framework\App\ObjectManager;
use Magento\Framework\Event\Observer;
use Magento\Framework\Event\ObserverInterface;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Bss\DependentCustomOption\Helper\DependOption;

class CatalogProductImportBunchSaveAfter implements ObserverInterface
{
    protected $productRepository;
    protected $dcoHelper;

    public function __construct(
        ProductRepositoryInterface $productRepository,
        DependOption $dcoHelper
    ) {
        $this->productRepository = $productRepository;
        $this->dcoHelper = $dcoHelper;
    }

    public function execute(Observer $observer)
    {
        try {
            $bunch = $observer->getBunch();
            foreach($bunch as $product) {

                if (isset($product['custom_options'])) {
                    $productModel = null;
                    if (is_array($product)) {
                        if (isset($product['id'])) {
                            $productModel = $this->productRepository->getById((int)$product['id']);
                        } elseif (isset($product['sku'])) {
                            $productModel = $this->productRepository->get($product['sku']);
                        }
                    } elseif (is_object($product) && $product instanceof \Magento\Catalog\Model\Product) {
                        $productModel = $product;
                    }
                    if ($productModel) {
                        // Do update depend co
                        $options = $productModel->getOptions();
                        $customOptionsUpdate = $product['custom_options'];
                        $customOptionsUpdate = explode(',', $customOptionsUpdate);
                        $name = $type = null;
                        $required = 0;
                        foreach ($customOptionsUpdate as $item) {
                            if (strpos($item, 'name=') === 0) {
                                $name = substr($item, 5);
                            } elseif (strpos($item, 'type=') === 0) {
                                $type = substr($item, 5);
                            } elseif (strpos($item, 'required=') === 0) {
                                $required = substr($item, 9);
                            } else {
                                continue;
                            }
                        }
                        foreach ($options as $option) {
                            if ($option->getTitle() == $name && $option->getType() == $type) {
                                $option->setBssDcoRequire($required);
                                $option->save();
                            }
                        }
                    }
                }
            }
        } catch (\Exception $e) {
            $logger = ObjectManager::getInstance()->get(\Psr\Log\LoggerInterface::class);
            $logger->error($e->getMessage());
        }
    }

}